import mediapipe as mp
from google.protobuf.json_format import MessageToDict
from ..processors import csv_gzip_writer as csv_writer

mp_faces = mp.solutions.face_mesh
mp_drawing = mp.solutions.drawing_utils
mp_drawing_styles = mp.solutions.drawing_styles

class MediapipeFace:
    def __init__(self, args):
        self.args = args

        self.faces = mp_faces.FaceMesh(
            static_image_mode=args.mp_faces_static == 'no',
            max_num_faces=args.mp_faces_max)

        if self.args.mp_faces_csv != None:
            self.writer = csv_writer.CsvGzipWriter(self.args.mp_faces_csv, self.args)

    def process(self, dispatched):
        results = self.faces.process(dispatched['frame_rgb'])
        if not results.multi_face_landmarks:
            return []

        records = []
        for face_index, face_landmarks_message in enumerate(results.multi_face_landmarks):
            face_landmarks = MessageToDict(face_landmarks_message)['landmark']

            if self.args.mp_faces_draw == 'yes':
                self.draw(dispatched['frame'], face_landmarks_message)

            record = self.organize(
                face_index,
                face_landmarks,
                dispatched['frame'],
                dispatched['frame_number'])

            records.append(record)

        if hasattr(self, 'writer'):
            for record in records:
                self.writer.write(record)

        return records

    def draw(self, frame, face_landmarks_message):
        mp_drawing.draw_landmarks(
            image=frame,
            landmark_list=face_landmarks_message,
            connections=mp_faces.FACEMESH_TESSELATION,
            landmark_drawing_spec=None,
            connection_drawing_spec=mp_drawing_styles.get_default_face_mesh_tesselation_style())

        mp_drawing.draw_landmarks(
            image=frame,
            landmark_list=face_landmarks_message,
            connections=mp_faces.FACEMESH_CONTOURS,
            landmark_drawing_spec=None,
            connection_drawing_spec=mp_drawing_styles.get_default_face_mesh_contours_style())

    def organize(self, face_index, face_landmarks, frame, frame_number):
        height, width, _ = frame.shape

        record = {
            'face_index': str(face_index),
            'frame_number': str(frame_number),
        }

        bone_id = 0
        for face_landmark in face_landmarks:
            bone_id += 1

            for axe in face_landmark:
                if axe == 'y':
                    scale = height
                else:
                    scale = width

                coordinate_name = 'face_' + str(bone_id) + '_' + axe
                record[coordinate_name] = '{:-f}'.format(face_landmark[axe] * scale)

        return record

    def close_writer(self):
        if hasattr(self, 'writer'):
            self.writer.close()



import cv2

class FrameDispatcher:
    def __init__(self, args):
        self.frame_number = 0
        self.args = args
        print('The frame dispatcher will now start running with this arguments', self.args)

    def get_frame(self):
        if self.args.video != None:
            capture = cv2.VideoCapture(self.args.video)
        else:
            capture = cv2.VideoCapture(self.args.camera)

        while True:
            result, frame = capture.read()
            if result == False:
                break

            self.frame_number += 1

            if self.args.first != None and 'camera' not in self.args:
                if self.frame_number < self.args.first:
                    continue

            if self.args.last != None:
                if self.frame_number > self.args.last:
                    break

            if self.args.flip == 'yes':
                frame = cv2.flip(frame, 1)

            frame_rgb = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)

            frame_to_dispatch = {
                'args': self.args,
                'frame_number': self.frame_number,
                'frame': frame,
                'frame_rgb': frame_rgb
            }

            if self.args.first != None and 'camera' in self.args:
                if self.frame_number >= self.args.first:
                    yield frame_to_dispatch
            else:
                yield frame_to_dispatch

            if cv2.waitKey(1) & 0xFF == 27:
                break

        capture.release()
        cv2.destroyAllWindows()



import argparse
import cv2
from os import path
from . import frame_dispatcher
from .trackers import mediapipe_face, mediapipe_hand, mediapipe_pose

class Cli:
    def __init__(self):
        self.parser = argparse.ArgumentParser(
            description='Uses MediaPipe to generate CSV files from videos or camera live streams with the records of the tracked movements.')
        self.parser.add_argument('--video', help='[filename] Existing video to be tracked. If empty, --camera will be used')
        self.parser.add_argument('--camera', help='[int] Number that identifies the camera to use')
        self.parser.add_argument('--interval', help='[int] Number of records to store in memory before writing them')
        self.parser.add_argument('--first', help='[int] First frame to capture')
        self.parser.add_argument('--last', help='[int] Last frame to capture')
        self.parser.add_argument('--flip', help='[yes|no] Flip the image horizontally')
        self.parser.add_argument('--mp_hands_max', help='[int] Maximum number of hands to track with MediaPipe')
        self.parser.add_argument('--mp_hands_static', help='[yes|no] Tell MediaPipe to run a continuous detection of hands')
        self.parser.add_argument('--mp_hands_draw', help='[yes|no] Draw MediaPipe\'s records of tracked hands')
        self.parser.add_argument('--mp_hands_csv', help='[filename] Export a CSV file with MediaPipe\'s records of tracked hands')
        self.parser.add_argument('--mp_faces_max', help='[int] Maximum number of faces to track with MediaPipe')
        self.parser.add_argument('--mp_faces_static', help='[yes|no] Tell MediaPipe to run a continuous detection of faces')
        self.parser.add_argument('--mp_faces_draw', help='[yes|no] Draw MediaPipe\'s records of tracked faces')
        self.parser.add_argument('--mp_faces_csv', help='[filename] Export a CSV file with MediaPipe\'s records of tracked faces')
        self.parser.add_argument('--mp_poses_static', help='[yes|no] Tell MediaPipe to run a continuous detection of poses')
        self.parser.add_argument('--mp_poses_draw', help='[yes|no] Draw MediaPipe\'s records of tracked poses')
        self.parser.add_argument('--mp_poses_csv', help='[filename] Export a CSV file with MediaPipe\'s records of tracked poses')
        self.parse_arguments()

    def main(self):
        dispatcher = frame_dispatcher.FrameDispatcher(self.args)

        first_iteration = True
        for dispatched_frame in dispatcher.get_frame():
            if self.using_mp_hands():
                if first_iteration:
                    print(dispatched_frame['frame_number'])
                    mp_hand_tracker = mediapipe_hand.MediapipeHand(self.args)
                mp_hand_tracks = mp_hand_tracker.process(dispatched_frame)

            if self.using_mp_faces():
                if first_iteration:
                    mp_face_tracker = mediapipe_face.MediapipeFace(self.args)
                mp_face_tracks = mp_face_tracker.process(dispatched_frame)

            if self.using_mp_poses():
                if first_iteration:
                    mp_pose_tracker = mediapipe_pose.MediapipePose(self.args)
                mp_pose_tracks = mp_pose_tracker.process(dispatched_frame)

            cv2.imshow('Motion Tracker', dispatched_frame['frame'])
            first_iteration = False

        if self.using_mp_hands():
            mp_hand_tracker.close_writer()

        if self.using_mp_faces():
            mp_face_tracker.close_writer()

        if self.using_mp_poses():
            mp_pose_tracker.close_writer()

    def using_mp_hands(self):
        return self.args.mp_hands_draw == 'yes' or \
            self.args.mp_hands_csv != None

    def using_mp_faces(self):
        return self.args.mp_faces_draw == 'yes' or \
            self.args.mp_faces_csv != None

    def using_mp_poses(self):
        return self.args.mp_poses_draw == 'yes' or \
            self.args.mp_poses_csv != None

    def parse_arguments(self):
        self.args = self.parser.parse_args()
        self.validate()
        self.ensure_types()
        self.set_defaults()

    def string_represents_integer(self, string):
        try:
            int(string)
            return True
        except ValueError:
            return False

    def stop_because(self, reason):
        print("\n" + reason + "\n")
        print('Try calling the program with -h for help')
        exit()

    def assert_is_not_empty(self, argument_name):
        value = getattr(self.args, argument_name)

        if value == None:
            self.stop_because('A value is missing for --' + argument_name)

    def assert_is_integer(self, argument_name):
        value = getattr(self.args, argument_name)

        if value != None and not self.string_represents_integer(value):
            self.stop_because(
                'Invalid value received for --' + argument_name + '. ' +
                'Try with a valid integer number.')

    def assert_only_one_is_not_empty(self, argument_name_1, argument_name_2):
        empty_1 = getattr(self.args, argument_name_1) == None
        empty_2 = getattr(self.args, argument_name_2) == None

        if not empty_1 and not empty_2:
            self.stop_because(
                'Invalid value received for --' +
                argument_name_1 + ' and --' + argument_name_2 + '. ' +
                'Only one of them can be provided at a time.')
        elif empty_1 and empty_2:
            self.stop_because(
                'Invalid value received for --' +
                argument_name_1 + ' and --' + argument_name_2 + '. ' +
                'One of them must be defined.')

    def assert_file_is_writable(self, argument_name):
        file_name = getattr(self.args, argument_name)

        try:
            with open(file_name, 'w') as f:
                pass
        except IOError as x:
            self.stop_because(
                'Invalid value received for --' + argument_name + '. ' +
                'Check that the file path is writable.')

    def assert_is_optional_yes_or_no(self, argument_name):
        value = getattr(self.args, argument_name)

        if value != None and value not in ['yes', 'no']:
            self.stop_because(
                'Invalid value received for --' + argument_name + '. ' +
                'Try with either yes, or no.')

    def assert_file_exists(self, argument_name):
        value = getattr(self.args, argument_name)

        if value != None and not path.isfile(value):
            self.stop_because('Invalid value received for --' + argument_name)

    def validate(self):
        self.assert_only_one_is_not_empty('video', 'camera')
        self.assert_file_exists('video')
        self.assert_is_integer('camera')
        self.assert_is_integer('interval')
        self.assert_is_integer('first')
        self.assert_is_integer('last')
        self.assert_is_optional_yes_or_no('flip')

        self.assert_is_integer('mp_hands_max')
        self.assert_is_optional_yes_or_no('mp_hands_static')
        self.assert_is_optional_yes_or_no('mp_hands_draw')
        if self.args.mp_hands_csv != None:
            self.assert_file_is_writable('mp_hands_csv')

        self.assert_is_integer('mp_faces_max')
        self.assert_is_optional_yes_or_no('mp_faces_static')
        self.assert_is_optional_yes_or_no('mp_faces_draw')
        if self.args.mp_faces_csv != None:
            self.assert_file_is_writable('mp_faces_csv')

        self.assert_is_optional_yes_or_no('mp_poses_static')
        self.assert_is_optional_yes_or_no('mp_faces_draw')
        if self.args.mp_poses_csv != None:
            self.assert_file_is_writable('mp_poses_csv')

    def ensure_is_int(self, field):
        if field in self.args:
            value = getattr(self.args, field)
            if value != None:
                setattr(self.args, field, int(value))

    def ensure_types(self):
        self.ensure_is_int('camera')
        self.ensure_is_int('interval')
        self.ensure_is_int('first')
        self.ensure_is_int('last')
        self.ensure_is_int('mp_hands_max')
        self.ensure_is_int('mp_faces_max')

    def set_defaults(self):
        if self.args.video == None and self.args.camera == None:
            self.args.camera = 0

        if self.args.first == None:
            self.args.first = 1

        if self.args.flip == None:
            self.args.flip = 'yes' if self.args.video == None else 'no'

        if self.args.interval == None:
            self.args.interval = 25

        if self.using_mp_hands():
            if self.args.mp_hands_max == None:
                self.args.mp_hands_max = 2

            if self.args.mp_hands_static == None:
                self.args.mp_hands_static = 'no'

            if self.args.mp_hands_draw == None:
                self.args.mp_hands_draw = 'yes'

        if self.using_mp_faces():
            if self.args.mp_faces_max == None:
                self.args.mp_faces_max = 1

            if self.args.mp_faces_static == None:
                self.args.mp_faces_static = 'no'

            if self.args.mp_faces_draw == None:
                self.args.mp_faces_draw = 'yes'

        if self.using_mp_poses():
            if self.args.mp_poses_static == None:
                self.args.mp_poses_static = 'no'

            if self.args.mp_poses_draw == None:
                self.args.mp_poses_draw = 'yes'

def main():
    tracker = Cli()
    tracker.main()

if __name__ == '__main__':
    main()



import mediapipe as mp
from google.protobuf.json_format import MessageToDict
from ..processors import csv_gzip_writer as csv_writer

mp_hands = mp.solutions.hands
mp_drawing = mp.solutions.drawing_utils
mp_drawing_styles = mp.solutions.drawing_styles

class MediapipeHand:
    BONES = [
        'wrist', 'thumb_cmc', 'thumb_mcp', 'thumb_ip', 'thumb_tip',
        'index_finger_mcp', 'index_finger_pip', 'index_finger_dip', 'index_finger_tip',
        'middle_finger_mcp', 'middle_finger_pip', 'middle_finger_dip', 'middle_finger_tip',
        'ring_finger_mcp', 'ring_finger_pip', 'ring_finger_dip', 'ring_finger_tip',
        'pinky_mcp', 'pinky_pip', 'pinky_dip', 'pinky_tip']

    def __init__(self, args):
        self.args = args

        self.hands = mp_hands.Hands(
            static_image_mode=args.mp_hands_static == 'yes',
            max_num_hands=args.mp_hands_max)

        if self.args.mp_hands_csv != None:
            self.writer = csv_writer.CsvGzipWriter(self.args.mp_hands_csv, self.args)

        print('MediaPipe hand tracker will now start running with this arguments', self.args)

    def process(self, dispatched):
        results = self.hands.process(dispatched['frame_rgb'])
        if not results.multi_hand_landmarks:
            return []

        records = []
        for hand_id, hand in enumerate(results.multi_handedness):
            hand = MessageToDict(hand)['classification'][0]
            hand_landmarks_message = results.multi_hand_landmarks[hand_id]
            hand_landmarks = MessageToDict(hand_landmarks_message)['landmark']

            if self.args.mp_hands_draw == 'yes':
                self.draw(dispatched['frame'], hand_landmarks_message)

            record = self.organize(
                hand,
                hand_landmarks,
                dispatched['frame'],
                dispatched['frame_number'])

            records.append(record)

        if hasattr(self, 'writer'):
            for record in records:
                self.writer.write(record)

        return records

    def draw(self, frame, hand_landmarks_message):
        mp_drawing.draw_landmarks(
            frame,
            hand_landmarks_message,
            mp_hands.HAND_CONNECTIONS,
            mp_drawing_styles.get_default_hand_landmarks_style(),
            mp_drawing_styles.get_default_hand_connections_style())

    def organize(self, hand, hand_landmarks, frame, frame_number):
        height, width, _ = frame.shape

        record = {
            'frame_number': str(frame_number),
            'hand_index': str(hand['index']),
            'hand_score': '{:-f}'.format(hand['score']),
            'hand_handedness': hand['label'],
        }

        for bone_id, bone in enumerate(self.BONES):
            bone_landmarks = hand_landmarks[bone_id]
            bone_name = self.BONES[bone_id]

            for axe in ('x', 'y', 'z'):
                if axe == 'y':
                    scale = height
                else:
                    scale = width

                bone_landmark = bone_landmarks[axe] * scale
                record[bone_name + '_' + axe] = '{:-f}'.format(bone_landmark)

        return record

    def close_writer(self):
        if hasattr(self, 'writer'):
            self.writer.close()



import os
import pathlib
from setuptools import setup, find_packages

VERSION = '0.0.4.1'
DESCRIPTION = 'Generate data files from captured motion'
HERE = pathlib.Path(__file__).parent
LONG_DESCRIPTION = (HERE / "README.md").read_text()

setup(
    name='trucotrack',
    version=VERSION,
    author='Carlos Capote Pérez-Andreu',
    author_email='<carlos.capote@hawara.es>',
    description=DESCRIPTION,
    long_description_content_type='text/markdown',
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=['mediapipe', 'opencv-python'],
    entry_points={
        'console_scripts': [
            'trucotrack=src.cli:main',
        ]
    },
    keywords=['python', 'motion', 'motion capture', 'video',
        'stream', 'video stream', 'camera stream'],
    classifiers=[
        'Development Status :: 1 - Planning',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
        'Operating System :: Unix',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows'])



import mediapipe as mp
from google.protobuf.json_format import MessageToDict
from ..processors import csv_gzip_writer as csv_writer

mp_poses = mp.solutions.pose
mp_drawing = mp.solutions.drawing_utils
mp_drawing_styles = mp.solutions.drawing_styles

class MediapipePose:
    BONES = [
        'nose', 'left_eye_inner', 'left_eye', 'left_eye_outer',
        'right_eye_inner', 'right_eye', 'right_eye_outer',
        'left_ear', 'right_ear', 'mouth_left', 'mouth_right',
        'left_shoulder', 'right_shoulder', 'left_elbow', 'right_elbow',
        'left_wrist', 'right_wrist', 'left_pinky', 'right_pinky',
        'left_index', 'right_index', 'left_thumb', 'right_thumb',
        'left_hip', 'right_hip', 'left_knee', 'right_knee',
        'left_ankle', 'right_ankle', 'left_heel', 'right_heel',
        'left_foot_index', 'right_foot_index']

    def __init__(self, args):
        self.args = args

        self.poses = mp_poses.Pose(
            static_image_mode=args.mp_poses_static == 'no',
            smooth_landmarks=True)

        if self.args.mp_poses_csv != None:
            self.writer = csv_writer.CsvGzipWriter(self.args.mp_poses_csv, self.args)

    def process(self, dispatched_frame):
        frame = dispatched_frame['frame']
        frame_rgb = dispatched_frame['frame_rgb']
        frame_number = dispatched_frame['frame_number']

        results = self.poses.process(frame_rgb)
        if not results.pose_landmarks:
            return []

        pose_landmarks_message = MessageToDict(results.pose_landmarks)['landmark']

        records = []
        self.draw(frame, results.pose_landmarks)
        record = self.organize(pose_landmarks_message, frame, frame_number)
        records.append(record)

        if hasattr(self, 'writer'):
            for record in records:
                self.writer.write(record)

        return records

    def draw(self, frame, pose_landmarks_message):
        if self.args.mp_poses_draw == 'yes':
            mp_drawing.draw_landmarks(
                frame,
                pose_landmarks_message,
                landmark_drawing_spec=mp_drawing_styles.get_default_pose_landmarks_style())

    def organize(self, pose_landmarks, frame, frame_number):
        height, width, _ = frame.shape

        record = {
            'frame_number': str(frame_number),
        }

        for bone_id, bone in enumerate(self.BONES):
            bone_landmarks = pose_landmarks[bone_id]
            bone_name = self.BONES[bone_id]

            for axe in ('x', 'y', 'z'):
                if axe == 'y':
                    scale = height
                else:
                    scale = width

                bone_landmark = bone_landmarks[axe] * scale
                record[bone_name + '_' + axe] = '{:-f}'.format(bone_landmark)

        return record

    def close_writer(self):
        if hasattr(self, 'writer'):
            self.writer.close()



import gzip

class CsvGzipWriter:
    def __init__(self, file, args):
        self.file = gzip.open(file, 'wb')
        self.header_printed = False
        self.cached = []
        self.size = args.interval

    def write(self, dictionary):
        if len(self.cached) == 0:
            header = ','.join(dictionary.keys())
            self.write_header(header)

        self.cached.append(','.join(dictionary.values()))

        if len(self.cached) >= self.size:
            self.flush()

    def write_header(self, header):
        if not self.header_printed:
            self.file.write(bytearray(header + "\n", 'utf-8'))
            self.header_printed = True

    def flush(self):
        cached = self.cached.copy()
        self.cached = []
        for line in cached:
            self.file.write(bytearray(line + "\n", 'utf-8'))

    def close(self):
        self.flush()
        self.file.close()
